%
% cstot.m
%
% Steve Cox 1/11/03
%
% hybrid euler on the Conner-Stevens fiber equation
%
% usage:        cstot(rad,ell,N,dt,t1,t2,T,Iapp,pinc)
%
%               rad = fiber radius (cm)
%               ell = fiber length (cm)
%               N = number of patches
%               dt = timestep (ms)
%               t1 = start of current pulse (ms)
%               t2 = end of current pulse (ms)
%               T = stopping time (ms)
%               Iapp = size of current pulse (micro amps)
%               pinc = number of time steps between plots
%
% examples:     cstot(100,.01,50,0)
%

function cstot(N,dt,pinc,pq)

rad = 0.001;
rad = 0.0002;
ell = .5;
t1 = 2;
t2 = 8.5;
Iapp = 0.005;
Nt = ceil(12/dt);

C_M = 1;                % micro F / cm^2
R_2 = 0.034;            % k Ohm cm
G_i = rad/2/R_2;          % scaled axial conductance
dx = ell/(N-1);             % patch length
Area = 2*pi*rad*dx;          % patch surface area
x = dx/2:dx:ell-dx/2;   % vector of patch midpoints

Dc = 22e-10;   % cm^2/ms
Db = 5e-10;   % cm^2/ms
Dd = 5e-10;   % cm^2/ms
kpd = 0.1;     % 1/ms/microM
kmd = 0.4;     % 1/ms
kpb = 1.5e-3;
kmb = 0.3e-3;
k1 = 0.2;       % 1/ms/microM
k2 = 0.184;    %1/microM
BT = 100;        % microM
DT = 40;
F = 1e5;            % Faraday's constant

vK = -72;	% mV
vA = -75;	% mV
vNa = 55;	% mV
vCl = -60; 	% mV
vCa = 100;

GK = 20*0;	% mS/(cm)^2
GNa = 120*0;	% mS/(cm)^2
GNa = 110*0;	% mS/(cm)^2
GA = 47.7;      % mS/(cm)^2
GA = 40;      % mS/(cm)^2
GA = 20*0;
GCl = 0.3;	% mS/(cm)^2
%GCl = 1;	% mS/(cm)^2
GCa = 5*0;

e = ones(N,1);
X = 0:dx:ell;
%GA = GA*(1+4.25*X');
%GA = 20;
%GA = 20*(1+2*X');
%GCa = GCa*(1-X'/2);
%GCl = GCl*(1+X');

% set up the c-d-b (calcium - dye - buffer) system

Id = spdiags(e,0,N,N);
A1 = spdiags([e -2*e e],-1:1,N,N);
A1(1,1) = -1;
A1(N,N) = -1;
A1 = A1/(dx^2);
A = [Dc*A1-kpb*BT*Id-kpd*DT*Id kmd*Id kmb*Id; 
     kpd*DT*Id Dd*A1-kmd*Id 0*Id;
     kpb*BT*Id 0*Id Db*A1-kmb*Id];
A = spdiags(ones(3*N,1),0,3*N,3*N) - dt*A;
[L,U] = lu(A);

c = 0.05*e;          % mu M
d = kpd*DT*c./(kmd+kpd*c);
b = kpb*BT*c./(kmb+kpb*c);
y = [c; d; b];

v = -68*e;
n = an(v)./(an(v)+bn(v));
m = am(v)./(am(v)+bm(v));
h = ah(v)./(ah(v)+bh(v));
acs = ainf(v);
bcs = binf(v);
z = caminf(v);

rhs = zeros(N,1);

B = spdiags([-e 2*e -e], -1:1, N, N)/dx/dx;
B(1,1) = 1/dx/dx;
B(N,N) = 1/dx/dx;

t = 0;
tcnt = 0;
plot3(X',t*ones(N,1),v)
hold on

xind = N/10:N/10:9*N/10;
tcnt = 1;
vind(tcnt,:) = v(xind);
zind(tcnt,:) = z(xind);
cind(tcnt,:) = c(xind);
bind(tcnt,:) = b(xind);
dind(tcnt,:) = d(xind);
iAind(tcnt,:) = 0*b(xind);
icaind(tcnt,:) = 0*b(xind);
etime(tcnt) = 0;

for tcnt=2:Nt

      t = t + dt;

      Istim = 0;
      if ( (t>t1 & t<t2) | (t>20+t1 & t<20+t2) | (t>40+t1 & t<40+t2) )
         Istim = Iapp;
      end

      n = ( n + dt*an(v) )./(1 + dt*(an(v)+bn(v)) ); 
      m = ( m + dt*am(v) )./(1 + dt*(am(v)+bm(v)) ); 
      h = ( h + dt*ah(v) )./(1 + dt*(ah(v)+bh(v)) ); 

      acs = ( taua(v).*acs + dt*ainf(v) )./( taua(v) + dt );
      bcs = ( taub(v).*bcs + dt*binf(v) )./( taub(v) + dt );
      z = ( camtau(v).*z + dt*caminf(v) )./( camtau(v) + dt );

      cNa = GNa * m.^3 .* h;
      cK = GK * n.^4;
      cA = GA.*acs.^3 .* bcs;

      c = y(1:N);
%      cCa = GCa.*z.*hCa(c);

     cCa = GCa.*caminf(v);

      ed = C_M/dt + GCl + cNa + cK + cA + cCa;

      Bb = spdiags(ed,0,N,N) + G_i*B;

      rhs = v*C_M/dt + cNa*vNa + cK*vK + cA*vA + GCl*vCl + cCa*vCa;
      rhs(1) = rhs(1) + Istim/Area;

      v = Bb \ rhs;

      etime(tcnt) = t;
      vind(tcnt,:) = v(xind)';
      zind(tcnt,:) = z(xind)';

      ICa = cCa.*(v-vCa);
      IA = cA.*(v-vA);

      icaind(tcnt,:) = ICa(xind)';
      iAind(tcnt,:) = IA(xind)';

      c = y(1:N);
      d = y(N+1:2*N);
      b = y(2*N+1:3*N);
      tmpd = kpd*d.*c;
      tmpb = kpb*b.*c;
      c4 = c.^4;
      qup = k1*(c4-0.05^4)./(k2^4+c4);

      rhs = y + dt*[tmpb+tmpd-qup-ICa/(rad*F); -tmpd; -tmpb];

      y = U \ (L \ rhs);

      cind(tcnt,:) = y(xind)';
      dind(tcnt,:) = y(N+xind)';
      bind(tcnt,:) = y(2*N+xind)';

      if mod(tcnt,pinc) == 0
         plot3(X',t*ones(N,1),v)
         drawnow
      end

end

xlabel('x (cm)','fontsize',16)
ylabel('t (ms)','fontsize',16)
zlabel('v (mV)','fontsize',16)

if pq
print -dpsc wave
end

figure(2)
plot(etime,dind)
xlabel('t (ms)','fontsize',16)
ylabel('[CaB] (\mu M)','fontsize',16)

if pq
print -dpsc dind
end

figure(3)
plot(etime,cind)
xlabel('t (ms)','fontsize',16)
ylabel('[Ca] (\mu M)','fontsize',16)

if pq
print -dpsc cind
end

figure(4)
plot(etime,bind)
xlabel('t (ms)','fontsize',16)
ylabel('[CaE] (\mu M)','fontsize',16)

if pq
print -dpsc bind
end

figure(5)
%[ax,h1,h2] = plotyy(etime,vind,etime,zind);
plot(etime,vind);
hold on
bv = 16.3805;
plot(etime,bv*ones(size(etime)),'k')
hold off
%plot(ax,etime,caminf(vind),'--')
%hold off
xlabel('t (ms)','fontsize',16)
ylabel('v  (mV)','fontsize',16)
%ylabel(ax(1),'v  (mV)','fontsize',16)
%ylabel(ax(2),'z ','fontsize',16)
[vmax,vmaxind] = max(vind,[],1);
vmaxtime = etime(vmaxind)
[zmax,zmaxind] = max(zind,[],1);
zmaxtime = etime(zmaxind)

if pq
print -dpsc vind
end

figure(6)
plot(etime,icaind);
xlabel('t (ms)','fontsize',16)
ylabel('I_{Ca}  (\mu A/cm^2)','fontsize',16)
[imax,imaxind] = min(icaind,[],1);
imaxtime = etime(imaxind)

delta1 = zmaxtime - vmaxtime

delta2 = imaxtime - zmaxtime

if pq
print -dpsc icaind
end

if 1==0

figure(2)
plot(etime,bind(:,7))
hold on
plot(etime,cind(:,7),'--')
xlabel('t (ms)','fontsize',16)
ylabel('(\mu M)','fontsize',16)
legend('b(0.35,t)','c(0.35,t)','location','NorthWest')
hold off

figure(4)
[ax,h1,h2]=plotyy(etime,icaind(:,7),etime,vind(:,7));
%plot(etime,icaind(:,7))
xlabel('t (ms)','fontsize',16)
ylabel(ax(1),'I_{Ca}(0.35,t)  (\mu A/cm^2)','fontsize',16)
ylabel(ax(2),'v(0.35,t)  (mV)','fontsize',16)

end

bv = 16.3805;
bot = caminf(bv)*(bv-100);

for j=1:9,

    gcarec(j) = min(icaind(:,j))/bot;

end


figure(7)
plot(X',GCa)
hold on
plot(X(xind),gcarec,'x-')
hold off
xlabel('x (cm)','fontsize',16)
ylabel('g_{Ca} at several x','fontsize',16)

if pq
print -dpsc gcarec
end

% resolve the critical time when hCa is in

figure(8)
plot(etime(2:end)-dt/2,diff(log(-icaind(:,1)))/dt)
hold on
plot(etime(2:end)-dt/2,diff(log(hCa(cind(:,1))))/dt,'r')
%plot(etime(2:end)-dt/2,diff(log(1./(1+cind))))/dt)
hold off

figure(9)
plot(etime,iAind);
xlabel('t (ms)','fontsize',16)
ylabel('I_A  (\mu A/cm^2)','fontsize',16)

save csrun

return

vm0 = [-65 .02]';
%figure(6)
options = optimset('display','iter');

for j=1:9,

%    dica = (icaind(3:end,j)-icaind(1:end-2,j))/2/dt;
%    rat = dica./icaind(2:end-1,j);
%    ret = etime(2:end-1);
%    plot(ret,rat)

    disp(['j = ' num2str(j)])
    ts = input('t start: ');
    te = input('t end: ');
    jind = find(ts<etime & etime<te);
    gv0 = [8 linspace(-70,-60,length(jind))];
%    gv0 = [10 vind(jind,j)'];
    bgv = fminunc(@(gv) icafit(gv,dt,icaind(jind,j)),gv0,options);
    
    keyboard

end

function val = icafit(gv,dt,ica)
g = gv(1);
v = gv(2:end);
m = zeros(size(v));
m(1) = caminf(v(1));
for j=2:length(ica)
    m(j) = (camtau(v(j))*m(j-1)+dt*caminf(v(j)))/(dt+camtau(v(j)));
end
val = trapz((ica'-g*m.*(v-100)).^2)*dt;

function dvm = vmode(t,y,ret,rat)
irat = interp1(ret,rat,t);
v = y(1,:);
m = y(2,:);
dvm(1,1) = -(v-100).*(irat - caminf(v)./camtau(v)./m + 1./camtau(v));
dvm(2,1) = (caminf(v)-m)./camtau(v);


function val = an(v)
val = .02*(v+45.7)./(1-exp(-(v+45.7)/10));

function val = bn(v)
val = .25*exp(-0.0125*(v+55.7));

function val = am(v)
val = 0.38*(v+29.7)./(1-exp(-(v+29.7)/10));

function val = bm(v)
val = 15.2*exp(-0.0556*(v+54.7));

function val = ah(v)
val = 0.266*exp(-0.05*(v+48));

function val = bh(v)
val = 3.8./(1+exp(-(v+18)/10));

function val = ainf(v)
val = 0.0761*exp(0.0314*(v+94.22))./(1+exp(0.0346*(v+1.17)));
val = val.^(1/3);

function val = taua(v)
val = 0.3632 + 1.158./(1+exp(0.0497*(v+55.96)));

function val = binf(v)
val = 1./(1+exp(0.0688*(v+53.3)));
val = val.^4;

function val = taub(v)
val = 1.24 + 2.678./(1+exp(0.0624*(v+50)));

function y = caminf(v)
y = (1+tanh((v+1)/15))/2;

function y = camtau(v)
y = 0.01;

function val = hCa(c);
val = (0.1)./(0.1+c);
